#include <jstring.h>
#include "psdraw.h"
#include <jiostr.h>
#include <math.h>

DrawTo :: DrawTo()
  { TextHeight=12.; 
    AvgCharWid=TextHeight/2;     
    erasing=false;
    GenerateEPS=false;
    linewidth=-1;
    fontsize=-1;
  }

// returns true if setting is recognized
// assumes name is in lowercase
bool DrawTo :: Setting(string name, Coord value)
  {
    if (name=="textheight")
      {
        if (value > 0.)
          {
            TextHeight=value;
            AvgCharWid=TextHeight/2;
          }
        else
          cerr << "Illegal textheight value of " << value << " ignored.\n";
        return true;
      }
    else if (name=="eps")
      {
        GenerateEPS=(value >= .5);
        return true;
      }
    return false;
  }

void DrawTo :: SetBounds(Coord wid, Coord hi)
  {
    if (!GenerateEPS)
      {
        hi+=72;
        wid+=72;
      }
    cout << "%!PS-Adobe-2.0";
    if (GenerateEPS)
      cout << " EPSF-2.0";
    cout << "\n"
            "%%Creator: graphcover Version 0.15\n"
            "%%Orientation: Portrait\n"
            "%%BoundingBox: 0 0 " << wid << ' ' << hi << "\n"
            "%%Pages: " << (GenerateEPS ? 0 : 1) << "\n";
    if (!GenerateEPS)
      cout << "%%PageOrder: Ascend\n";
    cout << "%%BeginSetup\n"
            "%%EndSetup\n"
            "%%Magnification: 1.0000\n"
            "%%EndComments\n"
            "/draw {0 setgray} bind def\n"
            "/erase {1 setgray} bind def\n"
            "save\n"
         << (GenerateEPS ? 0 : 72) << ' ' << hi << " translate\n"
            "1 -1 scale\n"
            "/n  {newpath} bind def\n"
            "/cp {closepath} bind def\n"
            "/gr {grestore} bind def\n"
            "/gs {gsave} bind def\n"
            "/l  {lineto} bind def\n"
            "/m  {moveto} bind def\n"
            "/s  {stroke} bind def\n"
            "/sc {scale} bind def\n"
            "/sh {show} bind def\n"
            "/ff {findfont} bind def\n"
            "/sf {setfont} bind def\n"
            "/scf {scalefont} bind def\n"
            "/slw {setlinewidth} bind def\n"
            "%%EndProlog\n"
            "%%Page: 1 1\n";
  }

void DrawTo :: DrawLine(Coord p1[2], Coord p2[2], Coord r, bool erase)
  {
    r *= 2;
    if (fabs(linewidth-r)>.001)
      cout << r << " slw ";                 // setlinewidth
    if (erase!=erasing)
      {
        erasing=erase;
        if (erasing)
          cout << "erase ";                 // 1 setgray
        else
          cout << "draw ";                  // 0 setgray
      }
    cout << "n "                            // newpath
         << p1[0] << " " << p1[1] << " m "  // moveto
         << p2[0] << " " << p2[1] << " l "  // lineto
         << "cp s\n";                       // closepath and stroke
  }

void DrawTo :: DrawDisc(Coord c[2], Coord r, bool erase)
  {
    if (erase!=erasing)
      {
        erasing=erase;
        if (erasing)
          cout << "erase ";                 // 1 setgray
        else
          cout << "draw ";                  // 0 setgray
      }
    cout << "n "                            // newpath
         << c[0] << " " << c[1] << " "      // <center>
         << r << " 0 360 arc "              // <radius>
         << "cp fill\n";                    // closepath and fill
  }

void DrawTo :: DrawText(Coord leftbaseline[2], string text)
  { // uses TextHeight to set font size
    if (TextHeight != fontsize)
      {
        fontsize = TextHeight;
          // find font, scale font, set font
        cout << "/Times-Roman ff " << fontsize << " scf sf\n";
      }

    // replace \ with \\ so postscript doesn't weird out    
    int i;
    for (i=0; i<text.length(); ++i)
      if (text[i]=='\\')
        {
          text.insert(i, "\\");
          ++i;
        }
    // replace ( with \(
    // replace ) with \)
    for (i=0; i<text.length(); ++i)
      if (text[i]=='(' || text[i]==')')
        {
          text.insert(i, "\\");
          ++i;
        }

    cout << leftbaseline[0] << ' ' << leftbaseline[1] << " m "
            "gs 1 -1 sc (" << text << ") draw sh gr\n";
  }

void DrawTo :: Done()
  {
    cout << "restore\n";
    if (! GenerateEPS)
      cout << "showpage\n";
  }

