#include <jstring.h>
#include <vector>
#include <list>
#include "drawgraph.h"
#include "elttypes.h"

// ---------------------- Vertex member functions

void Vertex :: Set(Coord inP[3], string labeltext)
  {
    for (int i=0; i<3; ++i)
      P[i]=inP[i];
    label=labeltext;
    HaloDepth=inP[3]+epsilon;
    TextDepth=inP[3]-epsilon;
  }

void Vertex :: Render(Scene &scene)
  {
    DrawTo *dt=scene.GetDrawTo();
    Coord xy[2];
    Project(P, xy);

    scene << (new Disc(xy, setting.DiscRNormal, TextDepth))
          << (new Disc(xy, setting.DiscRThick, HaloDepth, true));

    if (setting.DrawBaseLabels)
      {
        xy[0] += dt->AvgCharWid/2 + setting.DiscRNormal*0.707;
        xy[1] += up*(dt->TextHeight/4 + setting.DiscRNormal*0.707);

        scene << (new Text(xy, TextDepth, label));
      }
  }

void CoverVertex :: Render(Scene &scene)
  {
    DrawTo *dt=scene.GetDrawTo();
    Coord xy[2];
    Project(P, xy);

    scene << (new Disc(xy, setting.DiscRNormal, TextDepth))
          << (new Disc(xy, setting.DiscRThick, HaloDepth, true));

    if (setting.DrawCoverLabels)
      {
        xy[0] += dt->AvgCharWid/2 + setting.DiscRNormal*0.707;
        xy[1] += up*(dt->TextHeight/4 + setting.DiscRNormal*0.707);

        scene << (new Text(xy, TextDepth, label));
      }
  }


void Vertex :: UpdateBounds(Graph &graph, DrawTo *dt)
  {
    for (int k=0; k<3; ++k)
      {
        Coord R=(k==1) ? 0 : setting.DiscRThick;
        graph.Bounds[0][k]=instl min(P[k] - R, graph.Bounds[0][k]);
        graph.Bounds[1][k]=instl max(P[k] + R, graph.Bounds[1][k]);
      }

    if (setting.DrawBaseLabels)
      {
        Coord X=P[0] + dt->AvgCharWid/2 + dt->AvgCharWid*label.length() + setting.DiscRNormal*0.707;
        Coord Z=P[2] + (dt->TextHeight*5/4 + setting.DiscRNormal*0.707);
        graph.Bounds[0][0]=instl min(X, graph.Bounds[0][0]);
        graph.Bounds[1][0]=instl max(X, graph.Bounds[1][0]);
        graph.Bounds[0][2]=instl min(Z, graph.Bounds[0][2]);
        graph.Bounds[1][2]=instl max(Z, graph.Bounds[1][2]);
      }
  }

void Vertex :: Translate(Coord t[3])
  {
    for (int k=0; k<3; ++k)
      P[k]+=t[k];
    HaloDepth += t[2];
    TextDepth += t[2];
  }

